/**
  This program tests your implementation of StaticAnalysis.java.
  When your implentation is correct, the output from this
  program should look exactly like the contents of Tests_output.txt.

  You should consider creating your own test expressions to make
  sure your implementation is correct.
*/
public class Tests
{
   public static void main(String[] args)
   {
      String[] programs =
      {
         "x",

         "( prog y )",

         "( prog " +
           "( var x 0 ) " +
           "( set x ( + x 1 ) ) )",

         "( prog " +
           "( var u 0 ) " +
           "( var u 2 ) )",

         "( prog " +
           "( set u ( + 0 false ) ) )",

         "( prog " +
           "( var x ( + x 1 ) ) )",  // can't have a recursive var declaration

         "( prog " +
           "( f a ) )",

         "( prog " +
           "( fun ( f x ) " +
               "( var w 10 ) " +  // local variable
               "( + x w ) ) " +
           "( var w 20 ) " +
           "( var x 30 ) " +
           "( f w ) )",

         "( prog " +
           "( var x 10 ) " +      // global variable x
           "( fun ( f x ) " +     // formal parameter x
               "( var x 20 ) " +  // local variable x
               "( + x x ) ) " +
           "( f 0 ) )",

         "( prog " +
           "( var x 10 ) " +      // global variable x
           "( var w 20 ) " +      // global variable w
           "( var z 30 ) " +      // global variable z
           "( fun ( f x ) " +     // formal parameter x
               "( var w 40 ) " +  // local variable w
               "( + x w z ) ) " +
           "( f w ) )",

         "( prog " +
           "( var z 10 ) " +       // global variable z
           "( fun ( f x ) " +      // formal parameter x
               "( var w 10 ) " +   // local variable w
               "( if true z ( while false ( + u w ) ) ) " +
               "( var z 20 ) " +   // local variable z
               "( f x ) ) " +      // recursive function
           "( var z w ) " +        // redefine z, w is undefined
           "( f z ) )",

         "( prog " +
           "( var x 0 ) " +
           "( set n 0 ) " +
           "( while ( < n 100000000 ) " +  // 100,000,000 iterations
               "( if ( < n 100000000 ) " +
                   "( set x ( + 1 x ) ) " +
                   "( set u ( + 1 u ) ) ) ) )",

         "( prog " +
           "( var i1 5 ) " +
           "( while ( > i2 0 ) " +
               "( print i1 ) " +
               "( set i3 ( - i3 1 ) ) ) )",

        "( prog " +
          "( fun ( g x ) ( + x 10 ) ) " +
          "( fun ( f h ) ( h 20 ) ) " +
          "( f g ) )",

        "( prog " +
          "( fun ( g x ) ( + x 10 ) ) " +
          "( var h g ) " +
          "( h 15 ) )",

         "( prog " +
           "( var a 1 ) " +
           "( var b 2 ) " +
           "( a b ) )", // static analysis cannot detect this error

        "( prog " +
          "( fun ( g x y z ) ( + x y z ) ) " +
          "( fun ( f u v ) ( * u v ) ) " +
          "( var a 10 ) " +
          "( var b 20 ) " +
          "( f ( g 1 b 3 ) ( + a b ) ) )",

        "( prog " +
          "( var y 3 ) " +
          "( fun ( f x ) ( + x y ) ) " +
          "( fun ( f u ) ( * u w ) ) " +
          "( fun ( h v ) ( && v u ) ) " +
          "( k f ) )",

         "( prog " +
           "( var x 20 ) " +
           "( fun ( w z ) ( + z x ) ) " +
           "( w z ) )"
      };

      int i = 0;
      for (i = 0; i < programs.length; i++)
      {
         System.out.println(i + " ==========================================");

         try // to parse the program.
         {
            // Build the abstract syntax tree that represents the program.
            Tree ast = Parse.parse( programs[i] );

            // Pretty-print the abstract syntax tree.
            System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );

            // Print the infix version of the program.
            System.out.println( AST2infix.ast2infix( ast ) + "\n" );

            // Do the static analysis of the AST.
            boolean analysisOK = StaticAnalysis.analyze( ast );

            if (analysisOK)
            try // to evaluate the program (interpret the AST).
            {
               Value value = Evaluate.eval( ast );
               System.out.println( "result = " + value + "\n" );
            }
            catch (EvalException e)
            {
             //System.out.println( e );
               e.printStackTrace();
            }

            if(true)
            try // to create dot and png files from the AST.
            {
               // Create an empty dot file.
               String baseName = String.format("Language_2.%02d", i);
               java.io.PrintWriter out = new java.io.PrintWriter(
                                            new java.io.File(baseName + ".dot") );
               // Put dot commands into the dot file
               out.println( Tree2dot.tree2dot(ast) + "\n" );
               out.close();
               // Create a command line for running dot.exe.
               String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                               "-Tpng",
                               baseName + ".dot",
                               "-o",
                               baseName + ".png"};
               // Execute the command line.
               java.lang.Runtime.getRuntime().exec(cmd);
            }
            catch (Exception e)
            {
               System.out.println( e );
             //e.printStackTrace();
            }
         }
         catch (ParseException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
      }
   }//main()

}//Tests
